from setuptools import setup, find_packages

setup(
    name='biomeanalyzer',
    version='0.0.2',
    python_requires='>=3.9',
    package_dir={'': 'src'},
    packages=find_packages('src'),
    package_data={'biomeanalyzer': ['data/rrnDB-5.8_pantaxa_stats_NCBI.tsv']},
    zip_safe=False,
    install_requires=['numpy', 'pandas', 'scipy'],
    author='',
    author_email='',
    description='BiomeAnalyzer is a Python package for analyzing microbiome data.',
)



from pathlib import Path
from biomeanalyzer.Normalization import normalize_data
import pandas as pd
import os


def load_data(path_dataset, path_metadata):
    # Determine the file extension and choose the appropriate pandas function
    def read_file(path, type_data):
        _, ext = os.path.splitext(path)
        if ext == '.xlsx':
            return pd.read_excel(path, engine='openpyxl', index_col=0 if type_data == 'dataset' else None)
        elif ext == '.xls':
            return pd.read_excel(path, engine='xlrd', index_col=0 if type_data == 'dataset' else None)
        else:
            return pd.read_csv(path, sep=";", index_col=0 if type_data == 'dataset' else None)

    dataset = read_file(path_dataset, type_data='dataset')
    metadata = read_file(path_metadata, type_data='metadata')

    return dataset, metadata


def prepare_data_novogene(df):
    df = df.reset_index(drop=True)

    df['#NAME'] = df[('Taxonomy')].apply(clean_taxonomy)
    df = df.set_index("#NAME")
    df = df.drop(columns=['Taxonomy'])

    df = df * 100
    return df


def clean_taxonomy(taxonomy):
    parts = taxonomy.split(";")

    if parts[-1] == 's__':
        parts[-1] = parts[-2].replace("g__", "") + " sp"

    for i in range(len(parts)):
        if "__" in parts[i]:
            parts[i] = parts[i].split("__")[1]

    return ";".join(parts)


#ADD COLUMN #CLASS
def add_treatments(data_df, meta_df):
    # Assuming treatments are to be used as values for the #CLASS row
    treatments = meta_df['Treatment'].values

    # Create a DataFrame for the #CLASS row
    class_df = pd.DataFrame([treatments], columns=data_df.columns)
    class_df.index = ['#CLASS']

    # Concatenate the #CLASS row DataFrame with the original DataFrame
    updated_df = pd.concat([class_df, data_df], axis=0)
    updated_df.index.name = '#NAME'
    return updated_df


def arrange_data(df, meta_df, normalize=True, lab="Novogene"):
    if lab == "Novogene":
        df = prepare_data_novogene(df)
    elif lab == "RTL":
        pass

    if normalize == True:
        data = normalize_data(df)
    else:
        data = df

    data = data.astype(float)
    data = add_treatments(data, meta_df)

    return data


#DATA TO CSV (MICROBIOME ANALYST)
def data_to_csv(df, meta_df, normalize=True, lab="Novogene"):

    data = arrange_data(df, meta_df, normalize, lab)

    desktop = Path.home() / "Desktop"

    pd.DataFrame.to_csv(data, desktop / "output_data.csv", sep=";", index=True)

    return f'Files saved in {desktop} as output_data.csv'



import subprocess
import pandas as pd
import os
import tempfile
import shutil
from pathlib import Path
from biomeanalyzer.Data_IO import arrange_data


def make_kronas(df, meta_df, normalize=True, lab="Novogene"):

    dataset = arrange_data(df, meta_df, normalize=normalize, lab=lab)

    temp_dir = tempfile.mkdtemp()  # Create a temporary directory to store the split files
    split_dataset(dataset, temp_dir)  # Split the dataset into multiple files
    temp_list = sorted([f for f in os.listdir(temp_dir) if f.endswith('.csv')])  # List files in the temporary directory .csv
    input_file_list_path = os.path.join(temp_dir, 'input_files.txt')  # Make the absolute path to the file list

    # Write the list of input files to a file with their full paths
    with open(input_file_list_path, 'w') as file_list:
        for file_name in temp_list:
            file_list.write(f"{os.path.join(temp_dir, file_name)}\n")

    command = 'ktImportText'  # Krona command to run
    desktop = Path.home() / "Desktop" # Get the desktop path
    output_file = 'output_krona.html'  # Output file name

    # Construct the full command to run
    full_command = [command, '-o', output_file]
    # Add the input files to the command
    with open(input_file_list_path, 'r') as file_list:
        input_files = file_list.read().splitlines()
        full_command.extend(input_files)

    # Execute the command
    try:
        subprocess.run(full_command, check=True, cwd=temp_dir)
        print(f"Command '{command}' executed successfully. Krona {output_file} created.")
    except subprocess.CalledProcessError as e:
        print(f"An error occurred while executing '{command}': {e}")
    except FileNotFoundError as e:
        print(f"An error occurred: {e}")

    # Move the Krona to the desktop
    try:
        output_file_path = os.path.join(temp_dir, output_file)
        desktop_output = desktop / output_file
        shutil.move(output_file_path, desktop_output)
        print(f"Krona moved to {desktop}")
    except Exception as e:
        print(f"An error occurred while moving the Krona to the desktop: {e}")

    # Delete the temporary directory
    shutil.rmtree(temp_dir)
    print(f"Temporary working directory {temp_dir} deleted")


def split_dataset(dataset, path):
    data_to_split = split_taxonomy(dataset)

    for col_name in data_to_split.columns[:-7]:
        df = data_to_split[[col_name, '#k', '#p', '#c', '#o', '#f', '#g']]
        df.to_csv(f"{path}/{col_name}.csv", sep="\t", index=False, header=False)


def split_taxonomy(dataset):
    data_copy = dataset.copy() # Copy the dataset to avoid modifying the original

    if '#CLASS' in dataset.columns:
        data_copy = data_copy.drop(index=["#CLASS"])  # get rid of the row #CLASS

    data_copy = data_copy.reset_index(drop=False)  # get the names of the microrganisms

    split_data = data_copy['#NAME'].str.split(';', expand=True)
    split_data = split_data.fillna('')
    split_data = split_data.rename(columns={0: '#k', 1: '#p', 2: '#c', 3: '#o', 4: '#f', 5: '#g', 6: '#s'})
    split_data = split_data.map(clean_taxons)

    data_copy.drop(columns=["#NAME"], inplace=True)

    krona_data = pd.concat([data_copy, split_data], axis=1)

    return krona_data


def clean_taxons(string):  # Auxiliary function to clean the taxons
    if string == "s__":
        string = "sp."
    elif "__" in string:
        return string.split("__")[1]
    return string
